jQuery(document).ready(function($){

    wbuGlobalQtyElement = null;


    wbuQtyChangeCart = function(qtyElement) {

        // ask user if they really want to remove this product
        if ( !wbuZeroQuantityCheck(qtyElement) ) {
            return false;
        }

        // when qty is set to zero, then trigger default woocommerce remove product action
        if ( qtyElement.val() == 0 ) {
            removeLink = qtyElement.closest('.cart_item').find('.product-remove a');
            removeLink.click();

            return false;
        }

        if ( wbuSettings.cart_ajax_method == 'simulate_update_button' ) {
            // new method
            wbuSimulateUpdateCartButtonClick(qtyElement);
        }
        else {
            // old ajax method
            wbuMakeAjaxCartUpdate(qtyElement);
        }

        wbuAfterCallUpdateCart(qtyElement);
        
        return true;
    };

    wbuMakeAjaxCartUpdate = function(qtyElement) {

        if ( typeof qtyElement.attr('name') == 'undefined' ) {
            return;
        }

        matches = qtyElement.attr('name').match(/cart\[(\w+)\]/);
        
        if ( matches === null ) {
            return;
        }
        
        form = qtyElement.closest('form');

        $("<input type='hidden' name='update_cart' id='update_cart' value='1'>").appendTo(form);
        $("<input type='hidden' name='is_wbu_ajax' id='is_wbu_ajax' value='1'>").appendTo(form);

        wbuGlobalQtyElement = qtyElement;

        cart_item_key = matches[1];
        form.append( $("<input type='hidden' name='cart_item_key' id='cart_item_key'>").val(cart_item_key) );

        // get the form data before disable button
        formData = form.serialize();

        $("button[name='update_cart']").val( wbuSettings.cart_updating_text ).prop('disabled', true);
        $("a.checkout-button.wc-forward").addClass('disabled').html( wbuSettings.cart_updating_text );

        $.post( form.attr('action'), formData, wbuAjaxCartUpdateCallback, 'json');
    };

    wbuAjaxCartUpdateCallback = function(resp) {
        $('.cart-collaterals').html(resp.html);
        
        wbuGlobalQtyElement.closest('.cart_item').find('.product-subtotal').html(resp.price);
        
        $('#update_cart').remove();
        $('#is_wbu_ajax').remove();
        $('#cart_item_key').remove();
        
        $("button[name='update_cart']").val(resp.update_label).prop('disabled', false);
        $("a.checkout-button.wc-forward").removeClass('disabled').html(resp.checkout_label);

        // fix to update "Your order" totals when cart is inside Checkout page
        if ( $( '.woocommerce-checkout' ).length ) {
            $( document.body ).trigger( 'update_checkout' );
        }

        $( document.body ).trigger( 'updated_cart_totals' );
        $( document.body ).trigger( 'wc_fragment_refresh' );
    };

    wbuSimulateUpdateCartButtonClick = function(qtyElement) {

        // deal with update cart button
        updateButton = $("button[name='update_cart']");
        updateButton.removeAttr('disabled')
                    .val( wbuSettings.cart_updating_text )
                    .prop('disabled', true);
        
        // this small delay has been added to fix bug on firefox
        setTimeout(function(){ updateButton.click(); }, 20);


        // change the Update cart button
        $("a.checkout-button.wc-forward").addClass('disabled')
                                         .html( wbuSettings.cart_updating_text );
    };
    
    wbuWhenCartUpdated = function(qtyElement) {
        // fix some event lost because the ajax reload
        wbuLockQtyInput();
        wbuAddItemRemoveEffect();
    };
    
    wbuAfterCallUpdateCart = function(qtyElement) {
        // this is an overridable function
    };
    
    wbuAddItemRemoveEffect = function() {
        // this is an overridable function
    };

    wbuZeroQuantityCheck = function(el_qty) {
        if ( wbuInfo.isCart && ( el_qty.val() == 0 ) && ( wbuSettings.confirmation_zero_qty === 'yes' ) ) {

            if ( !confirm( wbuSettings.zero_qty_confirmation_text ) ) {
                el_qty.val(1);
                return false;
            }
        }

        return true;
    };

    wbuListenChange = function() {

        if ( ( wbuInfo.isCart || wbuInfo.isShop ) && ( wbuSettings.enable_auto_update_cart == 'yes' ) ) {
            $(".qty").off('change.wbu_qty_change').on('change.wbu_qty_change').change(function(e) {
                return wbuQtyChangeCart( $(this) );
            });
        }
    };

    wbuCartDeleteEvent = function() {
    };

    wbuQtyButtons = function() {
        
        // listen to events
        $(document).on('click','.wbu-btn-inc', {} ,function(e){
            return wbuQtyButtonClick( $(this), 1 );
        });

        $(document).on('click','.wbu-btn-sub', {} ,function(e){
            return wbuQtyButtonClick( $(this), -1 );
        });
        
        wbuLockQtyInput();
    };

    wbuQtyButtonClick = function(element, factor) {
        inputQty = element.parent().find('.qty');
        newQty = parseInt(inputQty.val()) + factor;

        // respect the minimum value
        minAttr = inputQty.attr('min');
        if ( typeof minAttr !== typeof undefined && minAttr !== false && newQty < parseInt(minAttr) ) {
            return false;
        }

        // respect the max stock limit
        maxAttr = inputQty.attr('max');
        if ( typeof maxAttr !== typeof undefined && maxAttr !== false && newQty > parseInt(maxAttr) ) {
            return false;
        }

        // when using Select, check if new quantity exists in options list
        if ( inputQty.is('select') && ( inputQty.find('option[value="'+newQty+'"]').length === 0 ) ) {
            return false;
        }

        inputQty.val( newQty );
        inputQty.change();

        return false;
    };

    
    wbuLockQtyInput = function() {
        // lock quantity input
        if ( wbuSettings.qty_buttons_lock_input == 'yes' ) {
            $('.qty').attr('readonly', 'readonly')
                     .css('background-color', 'lightgray');
        }
    };
    
    wbuQtyOnShop = function() {
        if ( !wbuInfo.isShop || ( wbuSettings.enable_quantity_on_shop != 'yes' ) ) {
            return;
        }
        
        $("form.cart").on("change", ".qty", function() {
            $(this.form).find("[data-quantity]").data("quantity", this.value);
        });
        
        // commented because some users related issue with this (@webgorilla)
        // $(document.body).on("adding_to_cart", function() {
            // $("a.added_to_cart").remove();
        // });
    };
    
    wbuQtyOnCheckout = function() {

        if ( !wbuInfo.isCheckout || ( wbuSettings.checkout_allow_change_qty != 'yes' ) ) {
            return;
        }

        $("form.checkout").on("change", ".qty", function(){
            var data = {
                action: 'wbu_update_checkout',
                security: wc_checkout_params.update_order_review_nonce,
                post_data: $( 'form.checkout' ).serialize()
            };
            
            jQuery.post(wbuInfo.ajaxUrl, data, function( response ) {
                $( 'body' ).trigger( 'update_checkout' );
            });
        });
        
        // add Quantity title on table
        $('body').on('updated_checkout', function(){
            quantityTitle = '<div class="checkout-qty-title">' + wbuInfo.quantityLabel + '</div>';
            $('.shop_table th.product-name').append(quantityTitle);
        });
        
    };
    
    wbuProductAddToCartAjax = function() {
        if ( !wbuInfo.isSingleProduct ) {
            return;
        }

        $('.qty').on('change', function(){
            $('.add_to_cart_button').attr('data-quantity', $(this).val());
            $('[name="add-to-cart"]').attr('data-quantity', $(this).val());
        });
    };
    
    // onload function calls
    wbuListenChange();
    wbuQtyButtons();
    wbuCartDeleteEvent();
    wbuQtyOnShop();
    wbuQtyOnCheckout();
    wbuProductAddToCartAjax();
    
    // hide product quantity on cart if needed
    if ( wbuSettings.cart_hide_quantity == 'yes' ) {
        $('.product-quantity').parent().find('th').css('width', '100%');
        $('.product-quantity').hide();
    }

    if ( wbuInfo.isCart ) {
        // fix enter key problem
        if ( wbuSettings.cart_fix_enter_key == 'yes' ) {
            $('.woocommerce-cart-form .qty.text').on('keyup keypress', function(e) {
                var keyCode = e.keyCode || e.which;

                if (keyCode === 13) { 
                    $('.woocommerce-cart-form .qty.text').trigger( "blur" );
                    e.preventDefault();
                    return false;
                }
            });
        }

        // check hide update cart button
        if ( wbuSettings.cart_hide_update == 'yes' ) {
            $('button[name="update_cart"]').hide();
        }
    }

    // listen when ajax cart has updated
    $(document).on('updated_wc_div', function(){
        wbuListenChange();
        wbuWhenCartUpdated();
    });
    
    // listen when product has added to cart
    $(document).on('added_to_cart', function(){
        // add delay to qty change event works
        setTimeout(function(){ wbuListenChange(); }, 500);
    });
});
