<?php
if ( !class_exists('WBU_Settings')) {
class WBU_Settings {
    function init_hooks() {
        add_filter('plugin_action_links_' . WBU_PLUGIN, array($this, 'settings_plugin_links') );
        add_action('admin_menu', array($this, 'settings_menu'), 59 );
    }
    
    function default_settings() {
        $arr = array(
            'enable_auto_update_cart' => 'yes',
            'show_qty_buttons' => 'no', // this is related to cart only
            'show_show_quantity_buttons' => 'no',
            'qty_buttons_lock_input' => 'no',
            'confirmation_zero_qty' => 'yes',
            'zero_qty_confirmation_text' => 'Are you sure you want to remove this item from cart?',
            'qty_as_select_cart' => 'no',
            'qty_as_select_product' => 'no',
            'qty_as_select_shop' => 'no',
            'qty_select_items' => 5,
            'cart_updating_text' => __('Updating...'),
            'products_per_row' => 4,
            'optimize_cart_other_pages' => 'no',
            'enable_direct_checkout' => 'no',
            'direct_checkout_add_cart_text' => __('Buy now'),
            'checkout_allow_change_qty' => 'no',
            'checkout_allow_delete' => 'no',
            'checkout_display_unit_price' => 'yes',
            'replace_view_cart_text' => __('View checkout'),
            'variations_in_popup' => 'no',
            'variations_popup_title' => __('Select product variations'),
            'variations_popup_exclude_image' => 'yes',
            'variations_popup_exclude_info' => 'yes',
            'enable_quantity_on_shop' => 'no',
            'shop_change_products_per_show' => 'no',
            'shop_products_per_row' => 4,
            'add_cart_countdown_seconds' => 10,
            'add_cart_countdown_text' => __('You will be redirected to cart in:'),
            'enable_add_to_cart_countdown' => 'no',
            'hide_shop_paginator' => 'no',
            'hide_shop_sorting' => 'no',
            'cart_fix_enter_key' => 'no',
            'cart_ajax_method' => 'make_specific_ajax',
            'product_ajax_add_to_cart' => 'no',
            'product_hide_price_variable' => 'no',
            'product_hide_price_grouped' => 'no',
            'product_max_qty' => '',
            'product_hide_quantity' => 'no',
            'cart_hide_quantity' => 'no',
            'cart_hide_update' => 'no',
        );

        return apply_filters('wbu_default_settings', $arr);
    }
    
    function settings_plugin_links($links) {
        $action_links = array(
            'settings' => sprintf('<a href="%s">%s</a>', $this->admin_url(), __('Settings')),
        );

        return array_merge( $action_links, $links );
    }
    
    function settings_menu( ) {
        $title = wbu()->title();

        add_submenu_page(
            'woocommerce',
            $title,
            $title,
            'manage_options',
            'woocommerce-better-usability',
            array($this, 'settings_page')
        );
    }
    
    function admin_js_file() {
        return ( defined('WP_DEBUG') && ( WP_DEBUG === true ) ) ? 'assets/admin-settings.js' : 'assets/admin-settings.min.js';
    }
    
    function settings_save() {

        $opts = $this->merge_settings_post();

        // treat boolean values
        foreach ( $this->default_settings() as $name => $val ) {
            if ( isset($opts[$name]) && isset($_POST[$name]) && in_array($val, array('yes', 'no'))) {
                $opts[$name] = wbu()->boolean_option($name);
            }
        }

        update_option('wbu_settings', $opts);
    }
    
    function merge_settings_post() {
        $defaultSettings = $this->default_settings();
        $allOptions = $this->all_options();

        foreach ( $_POST as $key => $val ) {
            if ( in_array($key, array_keys($defaultSettings)) ) {
                if ( is_numeric($defaultSettings[$key]) ) {
                    $allOptions[$key] = (int) sanitize_text_field($val);
                }
                else {
                    $allOptions[$key] = sanitize_text_field($val);
                }
            }
        }

        return $allOptions;
    }
    
    function save_option($option, $value) {
        $opts = $this->all_options();
        $opts[$option] = $value;

        update_option('wbu_settings', $opts);
    }
    
    function all_options() {
        return array_merge($this->default_settings(), get_option('wbu_settings', array()));
    }
    
    function admin_url() {
        return admin_url('admin.php?page=woocommerce-better-usability');
    }
    
    function admin_url_current() {
        return $this->tab_url($this->active_tab());
    }

    function tab_url($tab) {
        return $this->admin_url().'&tab='.$tab;
    }

    function setting_tab($tab, $label) {
        $class = ($tab == $this->active_tab()) ? 'nav-tab-active' : '';
        $tab = '<a href="'. $this->tab_url($tab) . '" class="nav-tab '.$class.'">'.$label.'</a>';

        return apply_filters('wbu_tab_link', $tab);
    }

    function active_tab() {
        $tab = basename(sanitize_file_name( isset($_GET['tab']) ? $_GET['tab'] : null ));

        if ( preg_match('/^tab-(.*)$/', $tab) ) {
            return $tab;
        }

        return 'tab-shop.php';
    }
    
    function admin_tab() {
        return esc_attr(str_replace('.php', '', $this->active_tab()));
    }
    
    function settings_page() {
        if ( !current_user_can( 'manage_options' ) ) {
            exit("invalid permissions");
        }

        // Save settings if data has been posted
        if ( ! empty( $_POST ) && check_admin_referer('wbu_nonce') ) {

            if ( sanitize_text_field($_POST['save']) == __( 'Save settings' ) ) {
                $this->settings_save();
            }
            else if ( sanitize_text_field($_POST['save']) == __( 'Reset all settings' ) ) {
                update_option('wbu_settings', $this->default_settings());
            }
        }
        
        $this->enqueue_admin_scripts();

        ?>
        <div class="wrap">
            <?= wbu()->lite_version_notice() ?>
            <div class="icon32">
            <br />
            </div>
            <h2 class="nav-tab-wrapper"><?php echo esc_attr(wbu()->title_full()) ?></h2>
            <nav class="nav-tab-wrapper woo-nav-tab-wrapper">
                <?= $this->setting_tab('tab-shop.php', __('Shop')) ?>
                <?= $this->setting_tab('tab-product.php', __('Product')) ?>
                <?= $this->setting_tab('tab-cart.php', __('Cart')) ?>
                <?= $this->setting_tab('tab-checkout.php', __('Checkout')) ?>
                <?= $this->setting_tab('tab-buy.php', __('Get PRO')) ?>
            </nav>
            <div class="wbu-settings-page">
                <?php $this->render_active_tab() ?>
            </div>
        </div>
        <?php
    }
    
    function render_active_tab() {
        switch($this->active_tab()) {
            case 'tab-shop.php':
                wbu()->shop->form_settings();
                break;
            
            case 'tab-product.php':
                wbu()->product->form_settings();
                break;
            
            case 'tab-cart.php':
                wbu()->cart->form_settings();
                break;
            
            case 'tab-checkout.php':
                wbu()->checkout->form_settings();
                break;
            
            case 'tab-buy.php':
                $this->render_buy_tab();
                break;
        }
    }
    
    function render_buy_tab() {
        ?>
        <div class="buy-div">
            <h1>
                <?= __('WooCommerce Better Usability PRO version') ?>
            </h1>

            <h3><?= __('Features included in PRO version') ?>:</h3>
            <ul class="wbu-features">
                <?= wbu()->list_features() ?>
            </ul>

            <h1>
                <a href="https://gumroad.com/l/rLol" target="_blank"><?= __('Go to buy page') ?> &rarr;</a>
            </h1>
        </div>
        <?php
    }
    
    function enqueue_admin_scripts() {
        wp_enqueue_style( 'wbu_admin_style', plugins_url('assets/admin-settings.css', WBU_ROOT_FILE));
        wp_enqueue_script( 'wbu_admin_script', plugins_url($this->admin_js_file(), WBU_ROOT_FILE));

        wp_localize_script('wbu_admin_script', 'tab_current', $this->admin_tab());
    }
    
    function form_header() {
        ?>
        <form method="post" id="mainform" action="" enctype="multipart/form-data">
            <?php wp_nonce_field('wbu_nonce'); ?>
        <?php
    }
    
    function form_footer() {
        ?>
            <hr/>
            <input name="save" class="button-primary" type="submit" value="<?= __( 'Save settings' ); ?>" />
            <input name="save" class="button" type="submit" value="<?= __( 'Reset all settings' ); ?>" onclick="return confirm('Are you sure?')"/>
        </form>
        <?php
    }
    
    static function checkbox($name, $label, $class = null) {
        $value = wbu()->option($name);
        
        ?>
        <input type="hidden" name="<?php echo $name ?>" value="0" />
        <label>
            <input type="checkbox"
                   <?php if ( $value === 'yes' ): ?>checked<?php endif; ?>
                   class="<?php echo $class ?>"
                   name="<?php echo $name ?>">
            <?php echo $label ?>
        </label>
        <?php
    }
    
    static function text($name, $label, $size) {
        $value = wbu()->option($name);
        
        ?>
        <?php echo $label ?>
        <input type="text" value="<?php echo $value; ?>" size="<?php echo $size; ?>" name="<?php echo $name; ?>">
        <?php
    }
    
    static function number($name, $label, $min, $max) {
        $value = wbu()->option($name);
        
        ?>
        <?php echo $label ?>
        <input type="number" value="<?php echo $value; ?>" name="<?php echo $name; ?>" min="<?php echo $min; ?>" max="<?php echo $max; ?>">
        <?php
    }
    
    static function select($name, $label, $options) {
        $value = wbu()->option($name);
        
        ?>
        <?php echo $label; ?>
        <select name="<?php echo $name ?>">
            <?php foreach ( (array) $options as $optVal => $optLabel ): ?>
                <option <?php if ( $value == $optVal ): ?>selected<?php endif; ?>
                        value="<?php echo $optVal ?>"><?php echo $optLabel; ?></option>
            <?php endforeach; ?>
        </select>
        <?php
    }
}
}
