<?php
if ( !class_exists('WBU_Shop')) {
class WBU_Shop {
    function init_hooks() {
        add_filter('woocommerce_product_add_to_cart_text', array($this, 'add_to_cart_text') );
        add_filter('woocommerce_add_to_cart_redirect', array($this, 'add_to_cart_redirect') );
        add_filter('woocommerce_loop_add_to_cart_link', array($this, 'shop_quantity_input'), 10, 2 );
        add_filter('loop_shop_columns', array($this, 'product_columns'), 10 );
        add_filter('woocommerce_get_script_data', array($this, 'cart_fragment_params') );
        add_filter('wc_get_template', array($this, 'wc_get_template'), 10, 2 );

        // try to identify if the current theme is Divi theme, so make compatibility for the "Allow change quantity on Shop" feature
        // dont know if this is the best way to check this, so please open new support if you have anything better
        if ( substr_count(strtolower(get_stylesheet()), 'divi') > 0 ) {
            add_action( 'woocommerce_after_shop_loop_item', 'woocommerce_template_loop_add_to_cart', 20 );
        }
    }
    
    function add_to_cart_text($text) {
        global $product;

        if ( wbu()->is_shop_loop() && wbu()->option('enable_direct_checkout') == 'yes' ) {
            if ( $product instanceof WC_Product_Simple ) {
                return wbu()->option('direct_checkout_add_cart_text');
            }
        }

        return $text;
    }
    
    function add_to_cart_redirect($url) {
        if ( wbu()->is_shop_loop() && wbu()->option('enable_direct_checkout') == 'yes' ) {
            return wc_get_checkout_url();
        }

        return $url;
    }
    
    function cart_fragment_params($params) {

        if ( wbu()->is_shop_loop() && wbu()->option('enable_direct_checkout') == 'yes' ) {
            $params['i18n_view_cart'] = wbu()->option('replace_view_cart_text');
        }

        return $params;
    }
    
    function product_columns($columns) {
        if ( wbu()->option('shop_change_products_per_show') == 'yes' ) {
            return wbu()->option('shop_products_per_row');
        }

        return $columns;
    }
    
    function shop_quantity_input($html, $product) {

        if ( !wbu()->is_shop_loop() ) {
            return $html;
        }

        $allowChangeQty = ( wbu()->option('enable_quantity_on_shop') == 'yes' ) &&
                            $product &&
                            $product->is_type('simple') &&
                            $product->is_in_stock() &&
                            $product->is_purchasable() &&
                            !$product->is_sold_individually();

        if ( $allowChangeQty ) {
            $useSelectField = ( wbu()->option('qty_as_select_shop') == 'yes' );
            
            $quantityField = wbu()->get_template('shop-qty-field.php', array(
                'product' => $product,
                'useSelectField' => $useSelectField,
                'inputValue' => apply_filters('wbu_shop_quantity_value', 1, $product),
            ));
            
            if ( wbu()->option('show_show_quantity_buttons') == 'yes' && $product->is_type('simple') ) {
                $qtyWithoutDiv = str_replace(array('<div class="quantity">', '</div>'), array('', ''), $quantityField);
                $quantityField = wbu()->get_template('qty-buttons-shop.php', array('input' => $qtyWithoutDiv));
            }
            
            $qtyAreaArgs = apply_filters('wbu_shop_quantity_area_args', array(
                'product' => $product,
                'allowChangeQty' => $allowChangeQty,
                'quantityField' => $quantityField,
                'class' => 'button alt ajax_add_to_cart add_to_cart_button product_type_simple',
            ));
            
            $html = wbu()->get_template('shop-qty-area.php', $qtyAreaArgs);
        }

        return $html;
    }
    
    function wc_get_template( $located, $template_name ) {    

        if ( !wbu()->is_shop_loop() ) {
            return $located;
        }
        
        if ( $template_name == 'loop/result-count.php' &&  wbu()->option('hide_shop_paginator') == 'yes' ) {
            $located = wbu()->template_path('empty.php');
        }
        else if ( $template_name == 'loop/orderby.php' && wbu()->option('hide_shop_sorting') == 'yes' ) {
            $located = wbu()->template_path('empty.php');
        }

        return $located;
    }
    
    function form_settings() {
        ?>
        <?php wbu()->settings->form_header() ?>

        <div class="conf">
            <?= WBU_Settings::checkbox('enable_quantity_on_shop', __('Allow to change product quantity on shop page'), 'check-subconf') ?>
            <div class="subconf">
                <?= WBU_Settings::checkbox('qty_as_select_shop', __('Show quantity as select instead of numeric field')) ?>
                <br/>
                <?= WBU_Settings::checkbox('show_show_quantity_buttons', __('Show -/+ buttons around item quantity')) ?>
                <?= do_action('wbu_settings_shop_qty_on_shop'); ?>
            </div>
        </div>
        <div class="conf">
            <?= WBU_Settings::checkbox('enable_direct_checkout', __('Go to checkout directly instead of cart'), 'check-subconf') ?>
            <div class="subconf">
                <?= WBU_Settings::text('direct_checkout_add_cart_text', __('Replace the default "Add to cart" link text to:'), 20) ?>
                <br/>
                <?= WBU_Settings::text('replace_view_cart_text', __('Replace the default "View cart" text to:'), 20) ?>
            </div>
        </div>
        <div class="conf">
            <?= WBU_Settings::checkbox('shop_change_products_per_show', __('Override the default number of products per row'), 'check-subconf') ?>
            <div class="subconf">
                <?= WBU_Settings::number('shop_products_per_row', __('Number of products to display per row:'), 1, 15) ?>
            </div>
        </div>
        <div class="conf">
            <?= WBU_Settings::checkbox('hide_shop_paginator', __('Hide product pagination')) ?>
        </div>
        <div class="conf">
            <?= WBU_Settings::checkbox('hide_shop_sorting', __('Hide the "Default sorting" select')) ?>
        </div>

        <?= do_action('wbu_settings_shop'); ?>
        <?php wbu()->settings->form_footer() ?>
        <?php
    }
}
}
