<?php
if ( !class_exists('WBU')) {
class WBU {
    /**
     * @var WBU_Settings
     */
    public $settings;
    
    /**
     * @var WBU_Cart
     */
    public $cart;
    
    /**
     * @var WBU_Checkout
     */
    public $checkout;

    /**
     * @var WBU_Shop
     */
    public $shop;
    
    /**
     * @var WBU_Product
     */
    public $product;
    
    function init_classes() {
        $this->settings = new WBU_Settings();
        $this->cart = new WBU_Cart();
        $this->checkout = new WBU_Checkout();
        $this->shop = new WBU_Shop();
        $this->product = new WBU_Product();
    }
    
    function init_hooks() {
        // general hooks
        add_action('init', array($this, 'plugin_init'));
        add_action('wp_ajax_wbu_dismissed_notice_handler', array($this, 'ajax_notice_handler') );
        add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts') );
        
        // specific hooks
        $this->settings->init_hooks();
        $this->shop->init_hooks();
        $this->cart->init_hooks();
        $this->checkout->init_hooks();
        $this->product->init_hooks();
    }

    function plugin_init() {

        if ( function_exists('is_shop') && wbu()->is_shop_loop() && wbu()->option('optimize_cart_other_pages') == 'yes' ) {
            wbu()->optimize_cart_on_shop();
        }
    }

    function list_features() {
        $readme = file_get_contents(WBU_ROOT_DIR . '/readme.txt');
        preg_match('/PRO version features:\n(.*)\n\n/isU', $readme, $match);

        $html = preg_replace(
            '/\[view demo\]\((.*)\)/',
            '&nbsp; <a href="$1" target="_blank">' . __('View demo') . ' <img src="'. plugins_url('assets/youtube.png', WBU_ROOT_FILE) .'"></a>',
            ( isset($match[1]) ?
              preg_replace('/\*(.*)/', '<li>$1</li>', $match[1]) :
              null )
        );
        
        return $html;
    }

    // this is custom code to cart page ajax work in pages like "Woocommerce Shop page", using the common WC shortcode [cart]
    function optimize_cart_on_shop() {
        wbu()->enqueue_cart_js();
    }

    function enqueue_cart_js() {
        $path = 'assets/js/frontend/cart.js';
        $src = str_replace( array( 'http:', 'https:' ), '', plugins_url( $path, WC_PLUGIN_FILE ) );

        $deps = array( 'jquery' );
        wp_enqueue_script( 'wc-cart', $src, $deps, WC_VERSION, true );
    }

    function js_uri() {
        $file = ( defined('WP_DEBUG') && ( WP_DEBUG === true ) ) ? 'assets/wbulite.js' : 'assets/wbulite.min.js';
        return apply_filters('wbu_js_uri', plugins_url($file, WBU_ROOT_FILE));
    }

    function css_uri() {
        return apply_filters('wbu_css_uri', plugins_url('assets/wbulite.css', WBU_ROOT_FILE));
    }

    // when render cart table page
    function enqueue_assets() {
        // enqueue assets
        wp_enqueue_script('wbulite', wbu()->js_uri(), array('jquery'));
        wp_enqueue_style('wbulite', wbu()->css_uri(), array('woocommerce-general'));

        wp_localize_script('wbulite', 'wbuSettings', wbu()->settings->all_options());
        wp_localize_script('wbulite', 'wbuInfo', array(
            'isCart' => is_cart(),
            'isShop' => wbu()->is_shop_loop(),
            'isSingleProduct' => is_product(),
            'isCheckout' => is_checkout(),
            'ajaxUrl' => get_admin_url() . 'admin-ajax.php',
            'quantityLabel' => __('Quantity'),
        ));
    }

    function is_shop_loop() {
        return ( is_shop() || is_product_category() );
    }

    function enqueue_scripts() {
        if ( is_checkout() || is_product() || wbu()->is_shop_loop() || is_cart() ) {
            wbu()->enqueue_assets();
        }
    }

    function template_path($name) {
        return apply_filters('wbu_get_template', WBU_ROOT_DIR . '/templates/' . $name);
    }
    
    function get_template($name, $args = array()) {
        if ( !empty($args) ) {
            extract($args);
        }
        
        ob_start();
        include $this->template_path($name);
        
        return ob_get_clean();
    }
    
    function title() {
        return apply_filters('wbu_menu_title', __( 'Better Usability' ));
    }

    function title_full() {
        return __('Woocommerce') . ' ' . wbu()->title();
    }

    function option($name) {
        $options = wbu()->settings->all_options();
        $value = isset($options[$name]) ? $options[$name] : null;

        return apply_filters('wbu_get_option', $value, $name);
    }

    function boolean_option($name) {
        return ( !empty($_POST[$name]) ? 'yes' : 'no' );
    }

    function show_notice($text, $type = null) {
        if ( !$type ) {
            $type = 'info';
        }

        $noticeHash = md5($text);

        // dont show when dismissed
        if ( wbu()->option('notice-dismiss-'.$noticeHash) ) {
            return;
        }

        ?>
        <div class="notice notice-wbu is-dismissible notice-<?= $type ?>" data-notice="<?= esc_attr($noticeHash) ?>">
            <p><?= $text ?></p>
        </div>
        <?php
    }

    function lite_version_notice() {
        if (  !apply_filters('wbu_lite_box', true) || ( wbu()->settings->active_tab() == 'tab-buy.php' ) ) {
            return;
        }

        $text = __("This is the lite version. If you want more advanced features, keep this plugin healthy and supported, please go to the <b>Get PRO</b> tab.");
        wbu()->show_notice($text);
    }

    // deprecated
    function form_checked($value) {
        return ( $value == 'yes' ) ? 'checked' : null;
    }

    function ajax_notice_handler() {
        if ( !current_user_can( 'manage_options' ) ) {
            exit("invalid permissions");
        }

        $noticeHash = sanitize_text_field($_POST['type']);
        wbu()->settings->save_option('notice-dismiss-'.$noticeHash, true);
    }
    
    function select_max_quantity() {
        $productMaxQty = wbu()->option('product_max_qty');
        
        return $productMaxQty > 0 ? $productMaxQty : wbu()->option('qty_select_items');
    }
    
    static function boot() {
        define('WBU_ROOT_DIR', plugin_dir_path( dirname(__FILE__)) );
        define('WBU_ROOT_FILE', dirname(dirname(__FILE__)) . '/' . basename(WBU_PLUGIN) );

        include_once( WBU_ROOT_DIR . 'includes/class-wbu-settings.php' );
        include_once( WBU_ROOT_DIR . 'includes/class-wbu-cart.php' );
        include_once( WBU_ROOT_DIR . 'includes/class-wbu-checkout.php' );
        include_once( WBU_ROOT_DIR . 'includes/class-wbu-shop.php' );
        include_once( WBU_ROOT_DIR . 'includes/class-wbu-product.php' );
        
        if ( !function_exists('wbu') ) {
            /**
             * @return WBU
             */
            function wbu() {
                static $instance = null;

                if ( !$instance ) {
                    $instance = new WBU();
                }

                return $instance;
            }
        }
        
        wbu()->init_classes();
        wbu()->init_hooks();
    }
}
}
